@@ -7,7 +7,7 @@ from __future__ import print_function
     This uses the Python type P4API.P4Adapter, which is a wrapper for the
     Perforce ClientApi object.
     
-    $Id: //depot/p15.2/p4-python/P4.py#1 $
+    $Id: //depot/r15.2/p4-python/P4.py#2 $
     
     #*******************************************************************************
     # Copyright (c) 2007-2010, Perforce Software, Inc.  All rights reserved.
@@ -580,6 +580,14 @@ class P4(P4API.P4Adapter):
             setattr(self, k, v)
                 
         flatArgs = self.__flatten(args)
+        
+        # if encoding is set, translate to Bytes
+        if hasattr(self,"encoding") and self.encoding and not self.encoding == 'raw':
+            result = []
+            for s in flatArgs:
+                result.append( s.encode(self.encoding) )
+            flatArgs = result
+        
         if self.logger:
             self.logger.info("p4 " + " ".join(flatArgs))
         
