@@ -27,7 +27,7 @@
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  * 
- * $Id: //depot/p15.2/p4-python/P4API.cpp#1 $
+ * $Id: //depot/r15.2/p4-python/P4API.cpp#2 $
  *
  * Build instructions:
  *  Use Distutils - see accompanying setup.py
@@ -220,7 +220,8 @@ static PyObject * P4Adapter_run(P4Adapter * self, PyObject * args)
     for (Py_ssize_t i = 1; i < PyTuple_Size(args); ++i) {
 	PyObject * item = PyTuple_GET_ITEM(args, i);
 #if PY_MAJOR_VERSION >= 3
-	if( ! PyUnicode_Check(item) ) {
+	// check encoding here?
+	if( ! PyUnicode_Check(item) && ! PyBytes_Check(item) ) { // preserve plain Unicode and Byte strings
 	    item = PyObject_Str(item);
 	}
 #else
