@@ -23,12 +23,12 @@
  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
- $Id: //depot/p15.2/p4-python/SpecMgr.cpp#1 $
+ $Id: //depot/r15.2/p4-python/SpecMgr.cpp#3 $
  *******************************************************************************/
 
 /*******************************************************************************
  * Name		: SpecMgr.cpp
- * 
+ *
  * Description	: Python bindings for the Perforce API. Class for handling
  * 		  Perforce specs. This class provides other classes with
  * 		  generic support for parsing and formatting Perforce
@@ -57,7 +57,7 @@ using namespace std;
 struct specdata {
     const char *type;
     const char *spec;
-}speclist[] = {
+} speclist[] = {
 
     {
 	"branch",
@@ -206,7 +206,7 @@ struct specdata {
 	"Files;code:460;len:8;;"
     },
     {
-	"prot",
+	"protect",
 	"Protections;code:501;fmt:C;type:wlist;words:5;opt:default;len:64;;"
     },
     {
@@ -225,6 +225,21 @@ struct specdata {
 	"DepotMap;code:860;type:wlist;words:2;len:64;;"
 	"ArchiveLimits;code:862;type:wlist;words:2;len:64;;"
     },
+    {
+	"server",
+	"ServerID;code:751;rq;ro;len:32;;"
+	"Type;code:752;rq;len:32;;"
+	"Name;code:753;type:line;len:32;;"
+	"Address;code:754;type:line;len:32;;"
+	"ExternalAddress;code:755;type:line;len:32;;"
+	"Services;code:756;rq;len:128;;"
+	"Description;code:757;type:text;len:128;;"
+	"User;code:761;type:line;len:64;;"
+	"ClientDataFilter;code:758;type:wlist;len:64;;"
+	"RevisionDataFilter;code:759;type:wlist;len:64;;"
+	"ArchiveDataFilter;code:760;type:wlist;len:64;;"
+	"DistributedConfig;code:762;type:text;len:128;;"
+    },
     {
 	"spec",
 	"Fields;code:351;type:wlist;words:5;rq;;"
@@ -256,11 +271,11 @@ struct specdata {
 	"ChangeView;code:714;type:llist;ro;len:64;;"
     },
     {
-	"trigger",
+	"triggers",
 	"Triggers;code:551;type:wlist;words:4;len:64;opt:default;"
     },
     {
-	"typeMap",
+	"typemap",
 	"TypeMap;code:601;type:wlist;words:2;len:64;opt:default;"
     },
     {
@@ -276,23 +291,7 @@ struct specdata {
 	"AuthMethod;code:662;fmt:L;len:10;val:perforce/ldap;;"
 	"Reviews;code:658;type:wlist;len:64;;"
     },
-    {
-	"server",
-	"ServerID;code:751;rq;ro;len:32;;"
-	"Type;code:752;rq;len:32;;"
-	"Name;code:753;type:line;len:32;;"
-	"Address;code:754;type:line;len:32;;"
-	"ExternalAddress;code:755;type:line;len:32;;"
-	"Services;code:756;rq;len:128;;"
-	"Description;code:757;type:text;len:128;;"
-	"User;code:761;type:line;len:64;;"
-	"ClientDataFilter;code:758;type:wlist;len:64;;"
-	"RevisionDataFilter;code:759;type:wlist;len:64;;"
-	"ArchiveDataFilter;code:760;type:wlist;len:64;;"
-	"DistributedConfig;code:762;type:text;len:128;;"
-    },
-
-    {   0, 0}
+    { 0, 0}
 };
 
 SpecMgr::SpecMgr(PythonDebug * dbg)
@@ -341,8 +340,8 @@ PyObject * SpecMgr::CreatePyStringAndSize( const char * text, size_t len ) {
 }
 
 //
-// Convert a Perforce StrDict into a Python dict. Convert multi-level 
-// data (Files0, Files1 etc. ) into (nested) array members of the dict. 
+// Convert a Perforce StrDict into a Python dict. Convert multi-level
+// data (Files0, Files1 etc. ) into (nested) array members of the dict.
 //
 
 PyObject * SpecMgr::StrDictToDict( StrDict *dict, PyObject * pydict) {
@@ -368,7 +367,7 @@ PyObject * SpecMgr::StrDictToDict( StrDict *dict, PyObject * pydict) {
 //
 
 PyObject * SpecMgr::StrDictToSpec( StrDict *dict, StrPtr *specDef ) {
-    // This converts it to a string, and then to a dict, so we go from one 
+    // This converts it to a string, and then to a dict, so we go from one
     // type of dictionary to another, via an intermediate form (a StrBuf).
 
     SpecDataTable dictData(dict);
@@ -434,7 +433,7 @@ PyObject * SpecMgr::StringToSpec( const char *type, const char *form, Error *e )
 }
 
 //
-// Format routine. updates a StrBuf object with the form content. 
+// Format routine. updates a StrBuf object with the form content.
 // The StrBuf can then be converted to a Python string where required.
 //
 void SpecMgr::SpecToString( const char *type, PyObject * pydict, StrBuf &b, Error *e ) {
@@ -459,7 +458,7 @@ void SpecMgr::SpecToString( const char *type, PyObject * pydict, StrBuf &b, Erro
 //
 // This method returns a dict describing the valid fields in the spec. To
 // make it easy on our users, we map the lowercase name to the name defined
-// in the spec (with an underscore in front). 
+// in the spec (with an underscore in front).
 // Thus, the users can always user lowercase, and if the field
 // should be in mixed case, it will be. See P4.Spec.__getattr__ and __setattr__
 //
